#include "IdleMode.h"

uint32_t PDMA_TEST_COUNT = 50;
uint8_t g_au32SrcArray0[8] = {0x1};
uint32_t g_u32DMAConfig = 0;

DMA_DESC_T DMA_DESC[2];

void IDLE_UARTFunction(void)
{

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART1                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART module */
    //SYS_ResetModule(UART1_RST);
    //UART_Open(UART1, 115200);

    /* Enable UART1 interrupt */
		UART0->WKCTL = UART_WKCTL_WKDATEN_Msk;
    NVIC_EnableIRQ(UART0_IRQn);
    UART_EnableInt(UART0, (UART_INTEN_WKIEN_Msk));

		CLK->PWRCTL |= CLK_PWRCTL_PDWKIEN_Msk;   
		NVIC_EnableIRQ(PWRWU_IRQn);

		UART0->WKSTS = UART0->WKSTS; // clecar status
	
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init EPWM0                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset EPWM0 module */
    SYS_ResetModule(EPWM0_RST);

    /* Set EPWM0 timer clock prescaler */
    EPWM_SET_PRESCALER(EPWM0, 0, 0);

    /* Set up counter type */
    EPWM0->CTL1 &= ~EPWM_CTL1_CNTTYPE0_Msk;

    // EPWM0 channel 0 frequency is 10Hz, duty 30%,
    EPWM_ConfigOutputChannel(EPWM0, 0, 10, 30);

    /* EPWM period point trigger ADC enable */
    EPWM_EnableADCTrigger(EPWM0, 0, EPWM_TRG_ADC_EVEN_PERIOD);

    /* Set output level at zero, compare up, period(center) and compare down of specified channel */
    EPWM_SET_OUTPUT_LEVEL(EPWM0, BIT0, EPWM_OUTPUT_HIGH, EPWM_OUTPUT_LOW, EPWM_OUTPUT_NOTHING, EPWM_OUTPUT_NOTHING);

    /* Enable output of EPWM0 channel 0 */
    EPWM_EnableOutput(EPWM0, BIT0);

		/*---------------------------------------------------------------------------------------------------------*/
    /* Init EADC                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
		/* Set input mode as single-end and enable the A/D converter */
		EADC_Open(EADC, EADC_CTL_DIFFEN_SINGLE_END);

		/* Configure the sample module 0 for analog input channel 5 and enable EPWM0 trigger source */
		EADC_ConfigSampleModule(EADC, 0, EADC_PWM0TG0_TRIGGER, 5);
	
		EADC->CTL &= 0xFFFFF3F; 
		EADC->CTL |= 1<<6; 
	
		EADC_ENABLE_PDMA(EADC);
	
		/* Enable EPWM0 channel 0 counter */
		EPWM_Start(EPWM0, BIT0); /* EPWM0 channel 0 counter start running. */	

		/*---------------------------------------------------------------------------------------------------------*/
    /* Init PDMA                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Open Channel 2, 4 */
    PDMA_Open(PDMA, 1 << 2);
	  PDMA_Open(PDMA, 1 << 4);

    /* Enable Scatter Gather mode, assign the first scatter-gather descriptor table is table 1,
       and set transfer mode as memory to peripheral */
		PDMA_SetTransferMode(PDMA, 2, PDMA_EADC0_RX, TRUE, (uint32_t)&DMA_DESC[1]);


    /* Scatter-Gather descriptor table configuration in SRAM */
    g_u32DMAConfig = \
                     (0 << PDMA_DSCT_CTL_TXCNT_Pos) | /* Transfer count is 1 */ \
                     PDMA_WIDTH_32 |  /* Transfer width is 32 bits(one word) */ \
                     PDMA_SAR_FIX |   /* Source increment size is fixed(no increment) */ \
                     PDMA_DAR_FIX |   /* Destination increment size is fixed(no increment) */ \
                     PDMA_REQ_SINGLE | /* Transfer type is burst transfer type */ \
                     PDMA_BURST_4 |   /* Burst size is 128. No effect in single transfer type */ \
                     PDMA_OP_SCATTER; /* Operation mode is scatter-gather mode */

    /*------------------------------------------------------------------------------------------------------
						Scatter Gather mode table 1
		------------------------------------------------------------------------------------------------------*/

    DMA_DESC[1].ctl = g_u32DMAConfig;
    /* Configure source address */
    DMA_DESC[1].src = EADC_BASE; /* Ping-Pong buffer 1 */
    /* Configure destination address */
    DMA_DESC[1].dest = UART0_BASE;
    /* Configure next descriptor table address */
    DMA_DESC[1].offset = (uint32_t)&DMA_DESC[1] - (PDMA->SCATBA); /* next operation table is table 2 */
		
//    /* Enable Scatter Gather mode, assign the first scatter-gather descriptor table is table 1,
//       and set transfer mode as memory to peripheral */
//    //PDMA_SetTransferMode(4, PDMA_MEM, TRUE, (uint32_t)&DMA_DESC[0]);
//		PDMA_SetTransferMode(4, PDMA_UART1_TX, TRUE, (uint32_t)&DMA_DESC[0]);
//		
//    /* Scatter-Gather descriptor table configuration in SRAM */
//    g_u32DMAConfig = \
//                     (0 << PDMA_DSCT_CTL_TXCNT_Pos) | /* Transfer count is 1 */ \
//                     PDMA_WIDTH_8 |  /* Transfer width is 32 bits(one word) */ \
//                     PDMA_SAR_FIX |   /* Source increment size is fixed(no increment) */ \
//                     PDMA_DAR_FIX |   /* Destination increment size is fixed(no increment) */ \
//                     PDMA_REQ_BURST | /* Transfer type is burst transfer type */ \
//                     PDMA_BURST_1 |   /* Burst size is 128. No effect in single transfer type */ \
//                     PDMA_OP_SCATTER; /* Operation mode is scatter-gather mode */

//    /*------------------------------------------------------------------------------------------------------
//						Scatter Gather mode table 1
//		------------------------------------------------------------------------------------------------------*/

//    DMA_DESC[0].ctl = g_u32DMAConfig;
//    /* Configure source address */
//    DMA_DESC[0].src = (uint32_t)(&g_au32SrcArray0[0]); /* Ping-Pong buffer 1 */
//    /* Configure destination address */
//    DMA_DESC[0].dest = UART1_BASE;
//    /* Configure next descriptor table address */
//    DMA_DESC[0].offset = (uint32_t)&DMA_DESC[0] - (PDMA->SCATBA); /* next operation table is table 2 */

//		UART1->INTEN |= UART_INTEN_TXPDMAEN_Msk | UART_INTEN_RXPDMAEN_Msk;

//    /* Start PDMA operation */
//    PDMA_Trigger(4);

    printf("\n\nCPU @ %dHz\n", SystemCoreClock);
    printf("+-----------------------------------------------------------------------+ \n");
    printf("|    M480 PDMA Driver Ping-Pong Buffer Sample Code (Scatter-gather)     | \n");
    printf("+-----------------------------------------------------------------------+ \n");

		//while(1);
		CLK_Idle();
//		
//		printf("Wake up from Idle mode");
//		
//		
//		UART_Close(UART1);
//		EPWM_Stop(EPWM0,BIT0);
//		EADC_Close(EADC);
//		PDMA_Close();
}
